import re

COUNTRY_CODES = {
    "93": "afghanistan",
    "355": "albania",
    "684": "american_samoa",
    "376": "andorra",
    "244": "angola",
    "1264": "anguilla_anguilla",
    "1268": "antigua_and_barbuda",
    "374": "armenia",
    "54": "argentina",
    "61": "australia",
    "43": "austria",
    "994": "azerbaijan",
    "1242": "bahamas",
    "973": "bahrain",
    "880": "bangladesh",
    "1246": "barbados",
    "375": "belarus",
    "32": "belgium",
    "501": "belize",
    "229": "benin",
    "1441": "bermuda",
    "591": "bolivia",
    "387": "bosnia_and_herzegovina",
    "267": "botswana",
    "55": "brazil",
    "1284": "british_virgin_islands_british_virgin_islands",
    "673": "brunei_darusalaam",
    "359": "bulgaria",
    "226": "burkina_faso",
    "257": "burundi",
    "855": "cambodia",
    "237": "cameroon",
    "238": "cape_verde",
    "1345": "cayman_islands",
    "236": "central_african_republic",
    "235": "chad",
    "56": "chile",
    "86": "china",
    "57": "colombia",
    "1670": "commonwealth_of_the_northern_mariana_islands",
    "269": "comoros_and_mayotte_island",
    "242": "congo",
    "682": "cook_islands",
    "506": "costa_rica",
    "385": "croatia",
    "53": "cuba",
    "357": "cyprus",
    "420": "czech_republic",
    "45": "denmark",
    "246": "diego_garcia",
    "253": "djibouti",
    "1767": "dominica",
    "1809": "dominican_republic",
    "593": "ecuador",
    "20": "egypt",
    "503": "el_salvador",
    "240": "equatorial_guinea",
    "372": "estonia",
    "251": "ethiopia",
    "298": "faeroe_islands",
    "500": "falkland_islands",
    "679": "fiji",
    "358": "finland",
    "33": "france",
    "590": "french_antilles",
    "594": "french_guiana",
    "689": "french_polynesia",
    "241": "gabon",
    "220": "gambia",
    "995": "georgia",
    "49": "germany",
    "233": "ghana",
    "350": "gibraltar",
    "30": "greece",
    "299": "greenland",
    "1473": "grenada",
    "671": "guam",
    "502": "guatemala",
    "224": "guinea",
    "245": "guinea",
    "592": "guyana",
    "509": "haiti",
    "504": "honduras",
    "852": "hong_kong",
    "36": "hungary",
    "354": "iceland",
    "91": "india",
    "62": "indonesia",
    "98": "iran",
    "964": "iraq",
    "353": "irish_republic",
    "972": "israel",
    "39": "italy",
    "225": "ivory_coast",
    "1876": "jamaica",
    "81": "japan",
    "962": "jordan",
    "254": "kenya",
    "686": "kiribati_republic",
    "996": "kirg",
    "965": "kuwait",
    "856": "laos",
    "371": "latvia",
    "961": "lebanon",
    "266": "lesotho",
    "231": "liberia",
    "370": "lithuania",
    "352": "luxembourg",
    "853": "macao",
    "389": "macedonia",
    "261": "madagascar",
    "265": "malawi",
    "60": "malaysia",
    "960": "maldives",
    "223": "mali",
    "356": "malta",
    "692": "marshall_islands",
    "596": "martinique",
    "222": "mauritania",
    "230": "mauritius",
    "52": "mexico",
    "691": "micronesia",
    "377": "monaco",
    "976": "mongolia",
    "1664": "montserrat",
    "212": "morocco",
    "213": "algeria",
    "258": "mozambique",
    "95": "myanmar",
    "264": "namibia",
    "674": "nauru",
    "977": "nepal",
    "31": "netherlands",
    "599": "netherlands_antilles",
    "687": "new_caledonia",
    "64": "new_zealand",
    "505": "nicaragua",
    "227": "niger",
    "234": "nigeria",
    "683": "niue",
    "672": "norfolk_island",
    "850": "north_korea",
    "967": "north_yemen",
    "670": "northern_mariana_islands",
    "47": "norway",
    "968": "oman",
    "92": "pakistan",
    "507": "panama",
    "675": "papua_new_guinea",
    "595": "paraguay",
    "51": "peru",
    "63": "philippines",
    "48": "poland",
    "351": "portugal",
    "1787": "puerto_rico",
    "974": "qatar",
    "378": "republic_of_san_marino",
    "262": "reunion",
    "40": "romania",
    "7": "russia",
    "77": "kazakhstan",
    "250": "rwandese_republic",
    "247": "saint_helena_and_ascension_island",
    "508": "saint_pierre_et_miquelon",
    "239": "sao_tome_e_principe",
    "966": "saudi_arabia",
    "221": "senegal",
    "248": "seychelles",
    "232": "sierra_leone",
    "65": "singapore",
    "421": "slovakia",
    "386": "slovenia",
    "677": "solomon_islands",
    "252": "somalia",
    "27": "south_africa",
    "82": "south_korea",
    "969": "south_yemen",
    "34": "spain",
    "94": "sri_lanka",
    "1869": "st.kitts_and_nevis",
    "1758": "st.lucia",
    "1784": "st.vincent_and_the_grenadines",
    "249": "sudan",
    "597": "suriname",
    "268": "swaziland",
    "46": "sweden",
    "41": "switzerland",
    "963": "syria",
    "992": "tajikistan",
    "886": "taiwan",
    "255": "tanzania",
    "66": "thailand",
    "228": "togolese_republic",
    "690": "tokelau",
    "676": "tonga",
    "1868": "trinidad_and_tobago",
    "21": "tunisia",
    "90": "turkey",
    "993": "turkmenistan",
    "1649": "turks",
    "688": "tuvalu",
    "1340": "us_virgin_islands",
    "256": "uganda",
    "380": "ukraine",
    "971": "united_arab_emirates",
    "44": "united_kingdom",
    "598": "uruguay",
    "1": "usa",
    "998": "uzbekistan",
    "678": "vanuatu",
    "58": "venezuela",
    "84": "vietnam",
    "681": "wallis_and_futuna_islands",
    "685": "western_samoa",
    "381": "yugoslavia",
    "243": "zaire",
    "260": "zambia",
    "263": "zimbabwe",
}
NOT_PHONE_SYMBOLS = re.compile(r'\D')


def guess_country_by_phone_number(phone_number: str) -> str | None:
    """
    Guesses the country of a phone number based on the given phone number.

    Parameters:
        phone_number (str): The phone number to guess the country of.

    Returns:
        str: The guessed country of the phone number, or None if the phone number is empty or no country is found.

    """
    phone_number = clean_phone_number(phone_number)

    if not bool(phone_number):
        return None

    for country_code in sorted(COUNTRY_CODES.keys(), key=lambda item: -int(item)):
        if phone_number.startswith(country_code):
            return COUNTRY_CODES[country_code]

    return None


def is_valid_phone_number(phone_number: str) -> bool:
    """

    This method checks if a given phone number is valid.

    Parameters:
        phone_number (str): The phone number to be checked.

    Returns:
        bool: True if the phone number is valid, False otherwise.

    """
    return bool(re.match(r"^\d{7,15}$", phone_number))


def clean_phone_number(phone_number: str) -> str:
    """
    Cleans a phone number by removing all non-digit characters and validates if the resulting cleaned number is valid.

    Parameters:
        phone_number (str): The input phone number to clean.

    Returns:
        str: The cleaned phone number if it is valid, otherwise an empty string.

    """
    result = re.sub(r'\D', '', phone_number)

    if not is_valid_phone_number(result):
        return ""

    return result



import json
from pathlib import Path


def read_json_file(file: str | Path) -> dict:
    """
    Reads and parses a JSON file.

    Parameters:
        file (str or Path): The path to the JSON file.

    Returns:
        dict: The parsed JSON data as dictionary.

    Raises:
        FileNotFoundError: If the specified file does not exist.
        ValueError: If the specified file is not a valid JSON file.

    """
    file = Path(file)

    if not file.is_file():
        raise FileNotFoundError(f'{file} does not exist')

    try:
        return json.loads(file.read_text(encoding='utf-8'))
    except json.JSONDecodeError:
        raise ValueError(f'{file} is not a valid JSON file')



from pathlib import Path

import pylogram

from ks_session_manager.env import env
from ks_session_manager.types import SessionMetadata
from ks_session_manager.types import SessionProxy


class PylogramClient(pylogram.Client):
    def __init__(
            self,
            data: SessionMetadata,
            *,
            session_path: str | Path | None = None,
            session_storage: pylogram.storage.Storage | None = None,
            proxy: SessionProxy | None = None,
            sign_in_enabled: bool = True,  # No-op
            ipv6: bool = False,
            test_mode: bool = False,
            bot_token: str = None,
            session_string: str = None,
            in_memory: bool = None,
            phone_code: str = None,
            password: str = None,
            workers: int = pylogram.Client.WORKERS,
            plugins: dict = None,
            parse_mode: pylogram.enums.ParseMode = pylogram.enums.ParseMode.HTML,
            no_updates: bool = None,
            takeout: bool = None,
            sleep_threshold: int = pylogram.session.Session.SLEEP_THRESHOLD,
            hide_password: bool = False,
            max_concurrent_transmissions: int = pylogram.Client.MAX_CONCURRENT_TRANSMISSIONS,
            ignore_channel_updates_except: list[int] = None,
            message_cache_size: int = 5000,
            first_name: str = None,
            last_name: str = None
    ):
        if not bool(session_path) and not bool(session_storage):
            raise ValueError("Either session_path or session_storage must be provided")

        if bool(session_path) and bool(session_storage):
            raise ValueError("Either session_path or session_storage must be provided, not both")

        if bool(session_path):
            session_path = Path(session_path).resolve()
            session_name = session_path.stem
            workdir = session_path.parent
        else:
            session_name = data.phone_number
            workdir = env.PYLOGRAM_WORKDIR

        super().__init__(
            session_name,
            phone_number=data.phone_number,
            password=password or data.two_factor_password,
            api_id=data.api_id,
            api_hash=data.api_hash,
            app_version=data.app_version,
            device_model=data.device_model,
            system_version=data.system_version,
            lang_code=data.lang_code,
            system_lang_code=data.system_lang_code,
            lang_pack=data.lang_pack,
            # system_lang_pack=data.system_lang_pack,
            proxy=proxy.to_pyrogram_format() if bool(proxy) else None,
            # Passthrough
            session_storage=session_storage,
            ipv6=ipv6,
            test_mode=test_mode,
            bot_token=bot_token,
            session_string=session_string,
            in_memory=in_memory,
            phone_code=phone_code,
            workers=workers,
            workdir=str(workdir),
            plugins=plugins,
            parse_mode=parse_mode,
            no_updates=no_updates,
            takeout=takeout,
            sleep_threshold=sleep_threshold,
            hide_password=hide_password,
            max_concurrent_transmissions=max_concurrent_transmissions,
            ignore_channel_updates_except=ignore_channel_updates_except,
            message_cache_size=message_cache_size,
            first_name=first_name,
            last_name=last_name
        )
        self.session_metadata: SessionMetadata = data
        self.sign_in_enabled = sign_in_enabled



class ConverterError(Exception):
    pass


class SigningInDisabledError(RuntimeError):
    def __init__(self, code_type: str):
        super().__init__("Session is not signed in")
        self.code_type = code_type



import abc

from ks_session_manager.clients import PylogramClient
from ks_session_manager.clients import TelethonClient
from ks_session_manager.types import SessionUserInfo


class BaseTelegramAdapter(abc.ABC):
    @property
    @abc.abstractmethod
    def is_connected(self) -> bool:
        raise NotImplementedError

    @abc.abstractmethod
    async def get_user_info(self) -> SessionUserInfo:
        raise NotImplementedError

    @abc.abstractmethod
    async def start(self):
        raise NotImplementedError

    @abc.abstractmethod
    async def stop(self):
        raise NotImplementedError


class TelethonTelegramAdapter(BaseTelegramAdapter):
    def __init__(self, client: TelethonClient):
        self.client = client

    @property
    def is_connected(self) -> bool:
        return self.client.is_connected()

    async def get_user_info(self) -> SessionUserInfo:
        if not self.is_connected:
            raise RuntimeError('Client is not connected')

        return SessionUserInfo(
            id=self.client.me.id,
            first_name=self.client.me.first_name,
            last_name=self.client.me.last_name,
            username=self.client.me.username,
            phone_number=self.client.me.phone,
            is_premium=self.client.me.premium,
        )

    async def start(self):
        # noinspection PyUnresolvedReferences
        await self.client.start()

    async def stop(self):
        await self.client.disconnect()


class PylogramTelegramAdapter(BaseTelegramAdapter):
    def __init__(self, client: PylogramClient):
        self.client = client

    @property
    def is_connected(self) -> bool:
        return self.client.is_connected

    async def get_user_info(self) -> SessionUserInfo:
        if not self.is_connected:
            raise RuntimeError('Client is not connected')

        return SessionUserInfo(
            id=self.client.me.id,
            first_name=self.client.me.first_name,
            last_name=self.client.me.last_name,
            username=self.client.me.username,
            phone_number=self.client.me.phone_number,
            is_premium=self.client.me.is_premium,
        )

    async def start(self):
        # noinspection PyUnresolvedReferences
        await self.client.start()

    async def stop(self):
        await self.client.disconnect()



import pathlib

import pydantic_settings


class Settings(pydantic_settings.BaseSettings):
    model_config = pydantic_settings.SettingsConfigDict(env_file=".env", extra='ignore')

    SMARTPROXY_USERNAME: str | None = None
    SMARTPROXY_PASSWORD: str | None = None
    SESSIONS_DIR: pathlib.Path = pathlib.Path.cwd() / '.sessions'

    # noinspection PyPep8Naming
    @property
    def PYLOGRAM_WORKDIR(self) -> pathlib.Path:
        return self.SESSIONS_DIR / 'pylogram'

    # noinspection PyPep8Naming
    @property
    def TELETHON_WORKDIR(self) -> pathlib.Path:
        return self.SESSIONS_DIR / 'telethon'


env = Settings()



import base64
from enum import StrEnum

import pydantic
from pydantic import ConfigDict
from pylogram.session.internals import DataCenter

from ks_session_manager.utils import phone


class PydanticBaseModel(pydantic.BaseModel):
    model_config = ConfigDict(
        str_strip_whitespace=True,
        populate_by_name=True,
        use_enum_values=True,
    )


class AccountType(StrEnum):
    TELETHON = 'TELETHON'
    PYLOGRAM = 'PYROGRAM'


class TelegramPlatform(StrEnum):
    IOS = 'IOS'
    MACOS = 'MACOS'
    ANDROID = 'ANDROID'
    DESKTOP_WINDOWS = 'DESKTOP_WINDOWS'
    DESKTOP_MACOS = 'DESKTOP_MACOS'
    DESKTOP_LINUX = 'DESKTOP_LINUX'
    WEB = 'WEB'


class SessionMetadata(PydanticBaseModel):
    user_id: int | None = pydantic.Field(0, alias='id')
    phone_number: str | None = pydantic.Field(None, alias='phone')
    two_factor_password: str | None = pydantic.Field(None, alias='twoFA')
    api_id: int = pydantic.Field(..., alias='app_id')
    api_hash: str = pydantic.Field(..., alias='app_hash')

    # Device settings
    app_version: str | None = None
    device_model: str | None = pydantic.Field(None, alias='device')
    system_version: str | None = pydantic.Field(None, alias='sdk')
    lang_code: str = 'en'
    system_lang_code: str = 'en-US'
    lang_pack: str | None = ''
    system_lang_pack: str | None = ''

    # Network settings
    ipv6: bool | None = False

    # Account info
    first_name: str | None = None
    last_name: str | None = None
    username: str | None = None
    avatar: str | None = None
    sex: int | None = None
    is_premium: bool | None = False

    # Timings
    register_time: int | float | None = None
    last_check_time: int | float | None = None

    # Tokens
    device_token: str | None = None
    device_token_secret: str | None = None
    device_secret: str | None = None
    signature: str | None = None
    certificate: str | None = None
    safetynet: str | None = None

    proxy: list[int | bool | str] | None = None

    @pydantic.field_validator('phone_number', mode='before')
    @classmethod
    def normalize_phone_number(cls, phone_number: str | None) -> str | None:
        return phone.clean_phone_number(phone_number) if phone_number else None


class SessionPeer(PydanticBaseModel):
    id: int
    access_hash: int
    phone_number: str | None = None
    username: str | None = None

    @property
    def peer_type(self) -> str:
        return 'bot' if bool(self.username) and self.username.endswith('bot') else 'user'


class SessionData(PydanticBaseModel):
    dc_id: int
    auth_key: bytes
    user_id: int | None = None
    is_bot: bool = False
    test_mode: bool = False
    server_address: str | None = None
    server_port: int | None = None
    takeout_id: int | None = None

    @pydantic.field_serializer('auth_key', return_type=str, when_used='json')
    def serialize_dt(self, auth_key: bytes, _info) -> str:
        return base64.standard_b64encode(auth_key).decode()

    @pydantic.model_validator(mode='after')
    def check_passwords_match(self) -> 'SessionData':
        if self.server_address is None or self.server_port is None:
            self.server_address, self.server_port = DataCenter(self.dc_id, self.test_mode, False, False)

        return self


class SessionProxy(PydanticBaseModel):
    scheme: str = "http"
    hostname: str
    port: int
    username: str | None = None
    password: str | None = None

    def to_telethon_format(self) -> dict:
        return {
            'proxy_type': self.scheme,
            'addr': self.hostname,
            'port': self.port,
            'username': self.username,
            'password': self.password,
        }

    def to_pyrogram_format(self) -> dict:
        return {
            'scheme': self.scheme,
            'hostname': self.hostname,
            'port': self.port,
            'username': self.username,
            'password': self.password
        }


class SessionUserInfo(PydanticBaseModel):
    id: int
    first_name: str
    last_name: str | None = None
    username: str | None = None
    phone_number: str | None = None
    is_premium: bool = False
